﻿#include "tools.h"
#include "../common/sources/Map.h"
#include "bridge.h"
#include <QDebug>

//-------------------------------------------------------------------
drawTileTool::drawTileTool(QUndoCommand *parent)
    : QUndoCommand(parent)
{
}

bool drawTileTool::init(CVec2 pos, int type)
{
    mPos = pos / TILE_SIZE;
    CTile*   tile = map.getTile(mPos);

    if (tile == NULL)
        return false;

    mOldType = tile->getType();

    if (mOldType == type)
        return false;

    mType = type;
    return true;
}

void drawTileTool::undo()
{
    map.setTile(mPos, mOldType);
    bridge.updateQMLImage();
}

void drawTileTool::redo()
{
    map.setTile(mPos, mType);
    bridge.updateQMLImage();
}

//-------------------------------------------------------------------
drawWallTool::drawWallTool(QUndoCommand *parent)
    : QUndoCommand(parent)
{
}

bool drawWallTool::init(CVec2 pos, int type)
{
    mPos = pos / TILE_SIZE;
    mSide = editor.getWallSideAbs(pos);

    CTile*   tile = map.getTile(mPos);

    if (tile == NULL)
        return false;

    mOldType = tile->mWalls[mSide].getType();

    if (mOldType == type)
        return false;

    mType = type;

    return true;
}

void drawWallTool::undo()
{
    map.setWall(mPos, mSide, mOldType);
    bridge.updateQMLImage();
}

void drawWallTool::redo()
{
    map.setWall(mPos, mSide, mType);
    bridge.updateQMLImage();
}

//-------------------------------------------------------------------
addObjectTool::addObjectTool(QUndoCommand *parent)
    : QUndoCommand(parent)
{
}

bool addObjectTool::init(CVec2 pos, int type)
{
    mPos = pos / (TILE_SIZE / 2);
    mType = type;
    return true;
}

void addObjectTool::undo()
{
    CObjectStack*   stack = map.findObjectsStack(mPos, eWallSideMax);

    if (stack != NULL)
    {
        size_t  size = stack->getSize();
        stack->removeObject(size - 1);

        if (size == 1)
            map.removeObjectsStack(mPos, eWallSideMax);
    }
    bridge.updateQMLImage();
}

void addObjectTool::redo()
{
    CObject newObject;
    newObject.setType(mType);
    CObjectStack*    stack = map.addObjectsStack(mPos, eWallSideMax);
    stack->addObject(newObject);
    bridge.updateQMLImage();
}

//-------------------------------------------------------------------
cutTool::cutTool(QUndoCommand *parent)
    : QUndoCommand(parent)
{
}

bool cutTool::init(CVec2 mousePos, ETabIndex tabIndex)
{
    if (editor.mClipboard.isEmpty() == true)
        return false;

    mPos = mousePos;
    mTabIndex = tabIndex;

    mData.copyFromClip(&editor.mClipboard);

    if (mData.isEmpty() == true)
        return false;

    return true;
}

void cutTool::undo()
{
    mData.copyToMap(mPos, mTabIndex);
    bridge.updateQMLImage();
}

void cutTool::redo()
{
    CVec2 pos = mPos / TILE_SIZE;

    if (mData.mType == eTabTiles)
    {
        if (mTabIndex == eTabTiles || mTabIndex == eTabWalls)
        {
            for (int y = 0; y < mData.mSize.y; ++y)
                for (int x = 0; x < mData.mSize.x; ++x)
                {
                    CTile*  tile = map.getTile(CVec2(pos.x + x, pos.y + y));

                    if (tile != NULL)
                    {
                        if (mTabIndex == eTabTiles)
                            tile->setType(0);
                        else if (mTabIndex == eTabWalls)
                            for (int i = 0; i < eWallSideMax; ++i)
                                tile->mWalls[i].setType(0);
                    }
                }

            if (mTabIndex == eTabWalls)
            {
                for (int y = 0; y < mData.mSize.y; ++y)
                    for (int x = 0; x < mData.mSize.x; ++x)
                        for (int side = 0; side < eWallSideMax; ++side)
                        {
                            CVec2   mapPos = pos + CVec2(x, y);
                            map.removeObjectsStack(mapPos, (EWallSide)side);
                        }
            }
        }
        else
        {
            for (int y = 0; y < mData.mSize.y * 2; ++y)
                for (int x = 0; x < mData.mSize.x * 2; ++x)
                {
                    CVec2   mapPos = pos * 2 + CVec2(x, y);
                    map.removeObjectsStack(mapPos, eWallSideMax);
                }
        }
    }
    else if (mData.mType == eTabWalls)
    {
        EWallSide   side = editor.getWallSideAbs(mPos);
        map.setWall(pos, side, 0);
        map.removeObjectsStack(pos, side);
    }
    else if (mData.mType == eTabObjects)
    {
        pos = mPos / (TILE_SIZE / 2);
        map.removeObjectsStack(pos, eWallSideMax);
    }

    bridge.updateQMLImage();
}

//-------------------------------------------------------------------
pasteTool::pasteTool(QUndoCommand *parent)
    : QUndoCommand(parent)
{
}

bool pasteTool::init(CVec2 mousePos, ETabIndex tabIndex)
{
    if (editor.mClipboard.isEmpty() == true)
        return false;

    mClipboard.copyFromClip(&editor.mClipboard);

    if (mClipboard.isEmpty() == true)
        return false;

    mPos = mousePos;
    mTabIndex = tabIndex;
    mData.copyFromMap(mousePos, mClipboard.mSize, tabIndex);

    switch (mClipboard.mType)
    {
    case eTabWalls: // un seul mur
        mData.mType = eTabWalls;
        mData.mSide = editor.getWallSideAbs(mousePos);
        break;

    case eTabObjects: // un seul objet
        mData.mType = eTabObjects;
        mData.mObjPos = (mousePos / (TILE_SIZE / 2)) % 2;
        break;

    default:
        break;
    }

//    if (mData.isEmpty() == true)
//        return false;

    return true;
}

void pasteTool::undo()
{
    mData.copyToMap(mPos, mTabIndex);
    bridge.updateQMLImage();
}

void pasteTool::redo()
{
    mClipboard.copyToMap(mPos, mTabIndex);
    bridge.updateQMLImage();
}
